<?php defined('SYSPATH') || die('No direct script access.');

class Controller_Handler_Journal extends Controller_Handler
{
    /** @var  Model_Rating */
    protected $model;

    public function before() {
        parent::before();

        $this->user->checkAccess(User::RIGHTS_TEACHER);
        $this->model = new Model_Journal;
    }

    public function action_SetAttendance() {
        $this->post
            ->rule('lessonID', 'digit')
            ->rule('recordBookID', 'digit')
            ->rule('attendance', 'numeric')
            ->rule('attendance', 'range', [':value', 0, 1]);
        if (!$this->post->check()) HTTP_API_Exception::factory(400, null);

        $error = Model_Journal::SetAttendance(
            $this->user->TeacherID,
            $_POST['recordBookID'],
            $_POST['lessonID'],
            $_POST['attendance']
        );
        if (!is_null($error)) throw HTTP_API_Exception::factory($error['code'], $error['message']);

        $this->response->body(json_encode([ 'success' => true ]));
    }

    public function action_CreateLesson() {
        $this->post
            ->rule('disciplineID', 'digit')
            ->rule('lessonType', 'digit');
            //->rule('date', 'date');
        if (!$this->post->check()) HTTP_API_Exception::factory(400, null);

        $result = Model_Journal::CreateLesson(
            $_POST['disciplineID'],
            $_POST['date'],
            $_POST['lessonType']
        );
        if (is_array($result)) throw HTTP_API_Exception::factory($result['code'], $result['message']);

        $this->response->body(json_encode([ 'success' => true, 'lessonID' => $result ]));
    }

    public function action_DeleteLesson() {
        $this->post
            ->rule('lessonID', 'digit');
        if (!$this->post->check()) HTTP_API_Exception::factory(400, null);

        $result = Model_Journal::DeleteLesson(
            $_POST['lessonID']
        );
        if (is_array($result)) throw HTTP_API_Exception::factory($result['code'], $result['message']);

        $this->response->body(json_encode([ 'success' => true ]));
    }

    public function action_SetLessonDate() {
        $this->post
            ->rule('lessonID', 'digit');
            //->rule('date', 'date');
        if (!$this->post->check()) HTTP_API_Exception::factory(400, null);

        $error = Model_Journal::SetLessonDate(
            $_POST['lessonID'],
            $_POST['date']
        );
        if (!is_null($error)) throw HTTP_API_Exception::factory($error['code'], $error['message']);

        $this->response->body(json_encode([ 'success' => true ]));
    }

    public function action_SetLessonName() {
        $this->post
            ->rule('lessonID', 'digit')
            #->rule('name', 'alpha_numeric')
             # match any whole string without special chars <>&$%#
            ->rule('name', 'regex', array(':value', "/^([^<>&$%#])*$/"))
            ->rule('name', 'max_length',array(':value', '200'));
        $checkResult = $this->post->check();
        if (!$checkResult) throw HTTP_API_Exception::factory(400, null);

        $error = Model_Journal::SetLessonName(
            $_POST['lessonID'],
            $_POST['name']
        );
        if (!is_null($error)) throw HTTP_API_Exception::factory($error['code'], $error['message']);

        $this->response->body(json_encode([ 'success' => true ]));
    }
}
